'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _Portal = require('./Portal');

var _Portal2 = _interopRequireDefault(_Portal);

var _styles = require('./styles');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /* eslint-disable no-undef */

var POSITION = {
    LEFT: 'left',
    RIGHT: 'right',
    TOP: 'top',
    CENTER: 'center',
    MIDDLE: 'middle',
    BOTTOM: 'bottom'
};

var Popover = function (_React$PureComponent) {
    _inherits(Popover, _React$PureComponent);

    function Popover(props) {
        _classCallCheck(this, Popover);

        var _this = _possibleConstructorReturn(this, (Popover.__proto__ || Object.getPrototypeOf(Popover)).call(this, props));

        _this.computePopoverPosition = function () {
            if (!_this.state.anchorElem) {
                return;
            }
            var boundingRect = _this.state.anchorElem.getBoundingClientRect();
            var top = boundingRect.top,
                left = boundingRect.left,
                height = boundingRect.height,
                width = boundingRect.width;

            var style = {};
            var separation = _this.props.separation || 0;
            var anchorOrigin = _this.props.anchorOrigin || {
                horizontal: POSITION.LEFT,
                vertical: POSITION.TOP
            };
            var targetOrigin = _this.props.targetOrigin || {
                horizontal: POSITION.LEFT,
                vertical: POSITION.TOP
            };

            if (anchorOrigin.horizontal === POSITION.LEFT) {
                style.left = left + 'px';
            } else if (anchorOrigin.horizontal === POSITION.MIDDLE) {
                style.left = left + width / 2 + 'px';
            } else if (anchorOrigin.horizontal === POSITION.RIGHT) {
                style.left = left + width + 'px';
            }

            if (anchorOrigin.vertical === POSITION.TOP) {
                style.top = top - separation + 'px';
            } else if (anchorOrigin.vertical === POSITION.CENTER) {
                style.top = top + height / 2 + 'px';
            } else if (anchorOrigin.vertical === POSITION.BOTTOM) {
                style.top = top + height + separation + 'px';
            }

            var transform = '';

            if (targetOrigin.horizontal === POSITION.MIDDLE) {
                transform += 'translateX(-50%) ';
            } else if (targetOrigin.horizontal === POSITION.RIGHT) {
                transform += 'translateX(-100%) ';
            }

            if (targetOrigin.vertical === POSITION.CENTER) {
                transform += 'translateY(-50%) ';
            } else if (targetOrigin.vertical === POSITION.BOTTOM) {
                transform += 'translateY(-100%) ';
            }

            style.transform = transform;

            return style;
        };

        _this.handleClickAwayLayer = function () {
            _this.setState({
                open: false
            });
            if (_this.props.onRequestClose) {
                _this.props.onRequestClose();
            }
        };

        _this.popoverRef = function (ref) {
            _this.popoverElem = ref;
            if (_this.props.popoverRef) {
                _this.props.popoverRef(ref);
            }
        };

        _this.state = {
            open: !!props.open,
            anchorElem: props.anchorElem || document.body
        };
        return _this;
    }

    _createClass(Popover, [{
        key: 'componentWillReceiveProps',
        value: function componentWillReceiveProps(nextProps) {
            if (nextProps.open !== this.props.open) {
                this.setState({
                    open: nextProps.open,
                    anchorElem: nextProps.anchorElem
                });
            }
        }

        /**
         * Computes the popover position with respect to the anchor element
         */


        /**
         * Hides the popover when click away layer is clicked
         */


        /**
         * Sets the popover reference
         */

    }, {
        key: 'render',
        value: function render() {
            var computedStyle = this.computePopoverPosition();
            var _props$clickAwayLayer = this.props.clickAwayLayer,
                clickAwayLayer = _props$clickAwayLayer === undefined ? true : _props$clickAwayLayer;

            var popover = _react2.default.createElement(
                _styles.Popover,
                {
                    innerRef: this.popoverRef,
                    style: computedStyle,
                    onClick: function onClick(e) {
                        return e.stopPropagation();
                    }
                },
                this.props.children
            );
            return _react2.default.createElement(
                _Portal2.default,
                { open: this.state.open },
                clickAwayLayer ? _react2.default.createElement(
                    _styles.OverlayPopup,
                    {
                        tabIndex: 0,
                        onClick: this.handleClickAwayLayer
                    },
                    popover
                ) : popover
            );
        }
    }]);

    return Popover;
}(_react2.default.PureComponent);

exports.default = Popover;


Popover.propTypes = {
    /**
     * If popover is open
     */
    open: _propTypes.PropTypes.bool,

    /**
     * Separtion (in pixels) between popover and anchor element
     */
    separation: _propTypes.PropTypes.number,

    /**
     * Sets the position of popover
     */
    anchorOrigin: _propTypes.PropTypes.shape({
        vertical: _propTypes.PropTypes.oneOf([POSITION.TOP, POSITION.CENTER, POSITION.BOTTOM]),
        horizontal: _propTypes.PropTypes.oneOf([POSITION.LEFT, POSITION.MIDDLE, POSITION.RIGHT])
    }),

    /**
     * Children
     */
    children: _propTypes.PropTypes.element.isRequired,

    /**
     * Sets the position of popover with respect to target
     */
    targetOrigin: _propTypes.PropTypes.shape({
        vertical: _propTypes.PropTypes.oneOf([POSITION.TOP, POSITION.CENTER, POSITION.BOTTOM]),
        horizontal: _propTypes.PropTypes.oneOf([POSITION.LEFT, POSITION.MIDDLE, POSITION.RIGHT])
    }),

    /**
     * Dummy layer which hides the popover when clicked on
     */
    clickAwayLayer: _propTypes.PropTypes.bool,

    /**
     * DOM Element reference for popover to open
     */
    anchorElem: _propTypes.PropTypes.element,

    /**
     * Callback for popover reference
     */
    popoverRef: _propTypes.PropTypes.func,

    /**
     * Callback when popover is closed
     */
    onRequestClose: _propTypes.PropTypes.func
};

Popover.defaultProps = {
    open: false,
    separation: 0,
    anchorOrigin: {
        vertical: POSITION.BOTTOM,
        horizontal: POSITION.LEFT
    },
    targetOrigin: {
        vertical: POSITION.TOP,
        horizontal: POSITION.LEFT
    },
    clickAwayLayer: true,
    anchorElem: null,
    popoverRef: null,
    onRequestClose: null
};